@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -1320,7 +1320,8 @@ brightonMidiInput(bristolMidiMsg *msg, guimain *global)
 			 * a couple of ints and a float. Should we use controller and
 			 * value?
 			 */
-			if (synth->win->template->callback != 0)
+			if ((msg->channel == synth->midichannel)
+				&& (synth->win->template->callback != 0))
                 synth->win->template->callback(synth->win,
 					msg->command, msg->params.program.p_id, 0);
 		}
@@ -1333,8 +1334,10 @@ brightonMidiInput(bristolMidiMsg *msg, guimain *global)
 	 * the value should be translated first.
 	bristolMidiToGM2(synth->win, msg);
 	 */
+//printf("PRE: %f\n", msg->params.controller.c_val);
 	bristolMidiToGM2(synth->win->GM2values, synth->win->midimap,
 		synth->win->valuemap, msg);
+//printf("POST: %f\n", msg->params.controller.c_val);
 
 	/* 
 	 * This should search the synth list, here we just assume one synth per
@@ -1626,7 +1629,8 @@ brightonMidiInput(bristolMidiMsg *msg, guimain *global)
 		 */
 		if (msg->GM2.c_id == 0)
 		{
-			if (synth->win->template->callback != 0)
+			if ((msg->channel == synth->midichannel)
+				&& (synth->win->template->callback != 0))
                 synth->win->template->callback(synth->win,
 					MIDI_BANK_SELECT, msg->GM2.intvalue, 0);
 			return;
