@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -1659,6 +1659,8 @@ modCallback(brightonWindow *win, int panel, int index, float value)
 		bristolMidiSendMsg(global.controlfd, synth->midichannel,
 			BRISTOL_EVENT_PITCH, 0, (int) (value * C_RANGE_MIN_1));
 	else {
+		if (synth->flags & NO_KEYTRACK)
+			return(0);
 		bristolMidiControl(global.controlfd, synth->midichannel,
 			0, 1, ((int) (value * (C_RANGE_MIN_1 - 1))) >> 7);
 	}
