@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -357,12 +357,12 @@ midiCallback(brightonWindow *win, int controller, int value, float n)
 {
 	guiSynth *synth = findSynth(global.synths, win);
 
-	printf("midi callback: %x, %i\n", controller, value);
+	//printf("midi callback: %x, %i\n", controller, value);
 
 	switch(controller)
 	{
 		case MIDI_PROGRAM:
-			printf("midi program: %x, %i\n", controller, value);
+			//printf("midi program: %x, %i\n", controller, value);
 			synth->location = value + synth->bank * 10;
 			if (loadMemory(synth, synth->resources->name, 0,
 				synth->location, synth->mem.active, FIRST_DEV, 0) < 0)
@@ -371,7 +371,7 @@ midiCallback(brightonWindow *win, int controller, int value, float n)
 				displayText(synth, "PRG", synth->location, FIRST_DEV + 59);
 			break;
 		case MIDI_BANK_SELECT:
-			printf("midi banksel: %x, %i\n", controller, value);
+			//printf("midi banksel: %x, %i\n", controller, value);
 			synth->bank = value;
 			synth->location = (synth->location % 10) + value * 10;
 			if (loadMemory(synth, synth->resources->name, 0,
