@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -32,10 +32,12 @@ static int pro1Init();
 static int pro1Configure();
 static int pro1Callback(brightonWindow *, int, int, float);
 /*static int keyCallback(void *, int, int, float); */
+static int proOneKeyCallback(brightonWindow *, int, int, float);
 /*static int modCallback(void *, int, int, float); */
 static int midiCallback(brightonWindow *, int, int, float);
 
 extern guimain global;
+static int seqLearn = 0;
 
 #include "brightonKeys.h"
 
@@ -339,7 +341,7 @@ brightonApp pro1App = {
 			0x020|BRIGHTON_STRETCH,
 			0,
 			0,
-			keyCallback,
+			proOneKeyCallback,
 			140, 680, 845, 300,
 			KEY_COUNT_3OCTAVE,
 			keys3octave
@@ -383,6 +385,52 @@ brightonApp pro1App = {
 	}
 };
 
+static void
+pro1SeqInsert(guiSynth *synth, int note, int layer)
+{
+	arpeggiatorMemory *seq = (arpeggiatorMemory *) synth->seq1.param;
+
+	if (seq->s_max == 0)
+		seq->s_dif = note + synth->transpose;
+
+	seq->sequence[(int) (seq->s_max)] =
+//		(float) (note + synth->transpose - seq->s_dif);
+		(float) (note + synth->transpose);
+
+//	if (pro10Debug(synth, 0))
+		printf("Seq put %i into %i\n", (int) seq->sequence[(int) (seq->s_max)],
+			(int) (seq->s_max));
+
+	if ((seq->s_max += 1) >= BRISTOL_SEQ_MAX)
+		seq->s_max = BRISTOL_SEQ_MAX;
+}
+
+static int
+proOneKeyCallback(brightonWindow * win, int panel, int index, float value)
+{
+	guiSynth *synth = findSynth(global.synths, win);
+
+	if (global.libtest)
+		return(0);
+
+//printf("proOneKeyCallback(%i, %i, %f) %i\n", panel, index, value, seqLearn);
+
+	if ((seqLearn) && (value != 0))
+		pro1SeqInsert(synth, index, panel);
+
+	/*
+	 * Want to send a note event, on or off, for this index + transpose.
+	 */
+	if (value)
+		bristolMidiSendMsg(global.controlfd, synth->midichannel,
+			BRISTOL_EVENT_KEYON, 0, index + synth->transpose);
+	else
+		bristolMidiSendMsg(global.controlfd, synth->midichannel,
+			BRISTOL_EVENT_KEYOFF, 0, index + synth->transpose);
+
+	return(0);
+}
+
 static int
 midiCallback(brightonWindow *win, int controller, int value, float n)
 {
@@ -419,13 +467,21 @@ pro1MidiSendMsg(void *synth, int fd, int chan, int c, int o, int v)
 static void
 pro1Sequence(guiSynth *synth, int fd, int chan, int c, int o, int v)
 {
+	if (synth->seq1.param == NULL)
+	{
+		/* This does not actually reload a sequence..... FFS, see Pro10 code */
+		loadSequence(&synth->seq1, "pro1", 0, 0);
+		fillSequencer(synth, (arpeggiatorMemory *) synth->seq1.param, 0);
+	}
 	switch (o) {
 		case 0:
 			/* On/Off */
 			if (v == 0) {
+				seqLearn = 0;
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_ENABLE, 0);
 			} else {
+				seqLearn = 0;
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_RESEQ, 0);
 				bristolMidiSendMsg(global.controlfd, synth->sid,
@@ -441,6 +497,7 @@ pro1Sequence(guiSynth *synth, int fd, int chan, int c, int o, int v)
 		case 1:
 			/* Record/Play */
 			if (v == 0) {
+				seqLearn = 0;
 				/* Turn off sequence learning */
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_RESEQ, 0);
@@ -458,6 +515,10 @@ pro1Sequence(guiSynth *synth, int fd, int chan, int c, int o, int v)
 					bristolMidiSendMsg(global.controlfd, synth->sid,
 						BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_ENABLE, 0);
 			} else {
+				arpeggiatorMemory *seq = (arpeggiatorMemory *) synth->seq1.param;
+				seqLearn = 1;
+				seq->s_max = 0;
+
 				bristolMidiSendMsg(global.controlfd, synth->sid,
 					BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_ENABLE, 0);
 				bristolMidiSendMsg(global.controlfd, synth->sid,
@@ -599,9 +660,13 @@ pro1Memory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 				synth->mem.active, 0, BRISTOL_FORCE) < 0)
 				displayText(synth, "FREE MEM",
 					synth->bank * 10 + synth->location, DISPLAY_DEV);
-			else
+			else {
 				displayText(synth, "PROGRAM",
 					synth->bank * 10 + synth->location, DISPLAY_DEV);
+				loadSequence(&synth->seq1,
+					"pro1", synth->bank * 10 + synth->location, 0);
+				fillSequencer(synth, (arpeggiatorMemory *) synth->seq1.param, 0);
+			}
 			synth->flags &= ~BANK_SELECT;
 			break;
 		case 2:
@@ -611,6 +676,7 @@ pro1Memory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 				synth->location = 1;
 			saveMemory(synth, "pro1", 0, synth->bank * 10 + synth->location,
 				0);
+			saveSequence(synth, "pro1", synth->bank * 10 + synth->location, 0);
 			displayText(synth, "PROGRAM", synth->bank * 10 + synth->location,
 				DISPLAY_DEV);
 			synth->flags &= ~BANK_SELECT;
@@ -623,9 +689,13 @@ pro1Memory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 						0, BRISTOL_STAT) < 0)
 					displayText(synth, "FREE MEM",
 						synth->bank * 10 + synth->location, DISPLAY_DEV);
-				else
+				else {
 					displayText(synth, "PROGRAM",
 						synth->bank * 10 + synth->location, DISPLAY_DEV);
+					loadSequence(&synth->seq1,
+						"pro1", synth->bank * 10 + synth->location, 0);
+					fillSequencer(synth, (arpeggiatorMemory *) synth->seq1.param, 0);
+				}
 			} else {
 				synth->bank = 0;
 				displayText(synth, "BANK", synth->bank, DISPLAY_DEV);
@@ -655,6 +725,8 @@ pro1Memory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 			synth->location = location;
 			loadMemory(synth, "pro1", 0, bank * 10 + location,
 				synth->mem.active, 0, BRISTOL_FORCE);
+			loadSequence(&synth->seq1, "pro1", bank * 10 + location, 0);
+			fillSequencer(synth, (arpeggiatorMemory *) synth->seq1.param, 0);
 			brightonParamChange(synth->win, 0,
 				MEM_START - 1 + synth->location, &event);
 			break;
@@ -683,6 +755,8 @@ pro1Memory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 				synth->mem.active, 0, BRISTOL_FORCE);
 			brightonParamChange(synth->win, 0,
 				MEM_START - 1 + synth->location, &event);
+			loadSequence(&synth->seq1, "pro1", bank * 10 + location, 0);
+			fillSequencer(synth, (arpeggiatorMemory *) synth->seq1.param, 0);
 			break;
 		case 6:
 			/* Find the next free mem */
@@ -1121,6 +1195,8 @@ printf("going operational\n");
 		synth->location %= 10;
 	}
 	loadMemory(synth, "pro1", 0, initmem, synth->mem.active, 0, 0);
+	loadSequence(&synth->seq1, "pro1", initmem, 0);
+	fillSequencer(synth, (arpeggiatorMemory *) synth->seq1.param, 0);
 
 	brightonPut(win,
 		"bitmaps/blueprints/pro1shade.xpm", 0, 0, win->width, win->height);
