@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -721,14 +721,15 @@ ummodCallback(brightonWindow *win, int panel, int index, float value)
 
 	synth->mem.param[UM_MOD_INDEX + index] = value;
 
-/*
+	/*
+	 * Selector buttonn debugs.
 	printf("ummodCallback(%i, %i, %f) [%f %f %f %f]\n", panel, index, value,
 		synth->mem.param[UM_MOD_INDEX + 2],
 		synth->mem.param[UM_MOD_INDEX + 3],
 		synth->mem.param[UM_MOD_INDEX + 4],
 		synth->mem.param[UM_MOD_INDEX + 5]
 		);
-*/
+	 */
 
 	/*
 	 * If this is controller 0 it is the frequency control, otherwise a 
@@ -874,6 +875,7 @@ printf("lmmodCallback(%i, %i, %f)\n", panel, index, value);
 				BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_ENABLE, 0);
 			bristolMidiSendMsg(global.controlfd, synth->sid,
 				BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_RESEQ, ivalue);
+			seqLearn = ivalue;
 			break;
 		case 11:
 			/* Upper manual Seq Play */
@@ -885,6 +887,7 @@ printf("lmmodCallback(%i, %i, %f)\n", panel, index, value);
 				BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_RESEQ, 0);
 			bristolMidiSendMsg(global.controlfd, synth->sid,
 				BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_ENABLE, ivalue);
+			seqLearn = 0;
 			break;
 		case 12:
 			/* Lower manual Seq Rec */
@@ -896,6 +899,7 @@ printf("lmmodCallback(%i, %i, %f)\n", panel, index, value);
 				BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_ENABLE, 0);
 			bristolMidiSendMsg(global.controlfd, synth->sid2,
 				BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_RESEQ, ivalue);
+			seqLearn = ivalue;
 			break;
 		case 13:
 			/* Lower manual Seq Play */
@@ -907,6 +911,7 @@ printf("lmmodCallback(%i, %i, %f)\n", panel, index, value);
 				BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_RESEQ, 0);
 			bristolMidiSendMsg(global.controlfd, synth->sid2,
 				BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_ENABLE, ivalue);
+			seqLearn = 0;
 			break;
 		case 14:
 			/* Seq/Arp rate */
@@ -960,7 +965,8 @@ printf("	Learning on second layer\n");
 		seq->s_dif = note + synth->transpose;
 
 	seq->sequence[(int) (seq->s_max)] =
-		(float) (note + synth->transpose - seq->s_dif);
+//		(float) (note + synth->transpose - seq->s_dif);
+		(float) (note + synth->transpose);
 
 //	if (pro10Debug(synth, 0))
 		printf("Seq put %i into %i\n", (int) seq->sequence[(int) (seq->s_max)],
@@ -1098,6 +1104,8 @@ int active, int skip, int flags)
 		synth = ((guiSynth *) synth->second);
 
 	loadMemory(synth, "prophet", 0, location, synth->mem.active, 0, 0);
+	loadSequence(&synth->seq1, "prophet", location, 0);
+	fillSequencer(synth, (arpeggiatorMemory *) synth->seq1.param, 0);
 
 	return(0);
 }
@@ -1203,9 +1211,13 @@ pro10MemoryLower(guiSynth *synth, int fd, int chan, int c, int o, int v)
 				synth->mem.active, 0, 0) < 0)
 				displayText(synth, "FRE", synth->bank + synth->location,
 					DISPLAY_DEV_2);
-			else
+			else {
 				displayText(synth, "PRG", synth->bank + synth->location,
 					DISPLAY_DEV_2);
+				loadSequence(&synth->seq1,
+					"prophet", synth->bank + synth->location, 0);
+				fillSequencer(synth, (arpeggiatorMemory *) synth->seq1.param, 0);
+			}
 			synth->flags &= ~BANK_SELECT;
 			break;
 		case 2:
@@ -1215,8 +1227,7 @@ pro10MemoryLower(guiSynth *synth, int fd, int chan, int c, int o, int v)
 				synth->location = 1;
 			saveMemory(synth, "prophet", 0, synth->bank + synth->location,
 				0);
-			saveSequence(synth, "prophet", synth->bank + synth->location,
-				BRISTOL_SID2);
+			saveSequence(synth, "prophet", synth->bank + synth->location, 0);
 			displayText(synth, "PRG", synth->bank + synth->location,
 				DISPLAY_DEV_2);
 			synth->flags &= ~BANK_SELECT;
@@ -1342,9 +1353,14 @@ pro10Memory(guiSynth *synth, int fd, int chan, int c, int o, int v)
 				synth->mem.active, 0, 0) < 0)
 				displayText(synth, "FRE", synth->bank + synth->location,
 					DISPLAY_DEV);
-			else
+			else {
 				displayText(synth, "PRG", synth->bank + synth->location,
 					DISPLAY_DEV);
+				loadSequence(&synth->seq1, "prophet",
+					synth->bank + synth->location, 0);
+				fillSequencer(synth,
+					(arpeggiatorMemory *) synth->seq1.param, 0);
+			}
 			synth->flags &= ~BANK_SELECT;
 			break;
 		case 2:
@@ -1986,6 +2002,8 @@ pro10Configure(brightonWindow *win)
 	synth->location = 1;
 	memForce(synth);
 	loadMemory(synth, "prophet", 0, initmem, synth->mem.active, 0, 0);
+	loadSequence(&synth->seq1, "prophet", initmem, 0);
+	fillSequencer(synth, (arpeggiatorMemory *) synth->seq1.param, 0);
 
 	synth->mem.param[PANEL_SWITCH] = 0;
 
@@ -1998,7 +2016,7 @@ pro10Configure(brightonWindow *win)
 	((guiSynth *) synth->second)->transpose = 12;
 
 	memForce(((guiSynth *) synth->second));
-	loadMemory(((guiSynth *) synth->second), "prophet", 0, 12,
+	loadMemory(((guiSynth *) synth->second), "prophet", 0, initmem  + 1,
 		synth->mem.active, 0, 0);
 
 	synth->mem.param[PANEL_SWITCH] = 0;
