@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -499,11 +499,15 @@ fillSequencer(guiSynth *synth, arpeggiatorMemory *mem, int flags)
 	if ((synth->flags & REQ_DEBUG_MASK) > REQ_MIDI_DEBUG2)
 		printf("Sending Arpeggiation to engine: %i events\n", (int) mem->s_max);
 
+	printf("fill sequence %i\n", (int) mem->s_max);
+
 	if (mem->s_max > 0)
 	{
 		bristolMidiSendMsg(global.controlfd, sid,
 			BRISTOL_ARPEGGIATOR, BRISTOL_SEQ_RESEQ, 1);
 
+		printf("send arpeggiation: %i\n", (int) mem->s_max);
+
 		for (i = 0; i < mem->s_max; i++)
 		{
 			printf("arpeggio step %i is %i\n", i, (int) mem->sequence[i]);
@@ -616,6 +620,7 @@ loadSequence(memory *seq, char *algo, int location, int flags)
 
 	close(fd);
 
+printf("loaded sequence\n");
 	return(0);
 }
 
@@ -812,6 +817,7 @@ saveSequence(guiSynth *synth, char *algo, int location, int flags)
 			return(-1);
 
 		if (checkSeqMem((arpeggiatorMemory *) seq->param) >= 0) {
+printf("Saving %i steps\n", (int) ((arpeggiatorMemory *) seq->param)->s_max);
 			if (write(fd, seq,
 				sizeof(struct Memory) - sizeof(float *)) < 0)
 				printf("seq write failed 1\n");
