@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -284,13 +284,16 @@ bristolArpeggiator(audioMain *audiomain, bristolMidiMsg *msg)
 			 * from the GUI memories.
 			 */
 			if (baudio->arpeggio.flags & BRISTOL_SEQ_LEARN) {
-				baudio->arpeggio.s.notes[baudio->arpeggio.s.max++].k = tval;
+				if (baudio->arpeggio.s.max == 0)
+					baudio->arpeggio.s.dif = tval;
+				baudio->arpeggio.s.notes[baudio->arpeggio.s.max].k
+					= tval - baudio->arpeggio.s.dif;
 				baudio->arpeggio.s.notes[baudio->arpeggio.s.max++].v = 120;
 				if (baudio->arpeggio.s.max > BRISTOL_SEQ_MAX)
 					baudio->arpeggio.s.max = BRISTOL_SEQ_MAX;
+//printf("insert %i\n", tval - baudio->arpeggio.s.dif);
 			}
 			break;
-
 		case BRISTOL_CHORD_ENABLE:
 			/* on/off */
 			if (value != 0)
@@ -495,6 +498,7 @@ bristolArpeggiatorNoteEvent(Baudio *baudio, bristolMidiMsg *msg)
 		}
 		arpeggio->s.notes[arpeggio->s.max].k
 			= msg->params.key.key - arpeggio->s.dif;
+//printf("live insert %i\n", arpeggio->s.notes[arpeggio->s.max].k);
 		arpeggio->s.notes[arpeggio->s.max].v
 			= msg->params.key.velocity - arpeggio->s.vdif;
 		if (++arpeggio->s.max >= BRISTOL_SEQ_MAX)
