@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
 #include <assert.h>
 
 #include "bristol.h"
-#include "bristolmm.h"
+//#include "bristolmm.h"
 
 #include "dco.h"
 
@@ -247,6 +247,12 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 		 */
 		while (v != NULL)
 		{
+#ifdef NOT_ANDROID_NATIVE_LIBRARY
+#warning ANDROID PATCH, MAY HAVE TO BACK IT OUT
+			if (v->flags & (BRISTOL_KEYON|BRISTOL_KEYREON))
+				v->flags &= ~BRISTOL_KEYDONE;
+			else
+#endif
 			if (v->flags & BRISTOL_KEYDONE)
 			{
 //printf("remove %x->%x/%x (0x%04x)\n", (size_t) v, (size_t) v->last,
@@ -264,6 +270,7 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 					continue;
 				}
 
+				v->flags &= ~BRISTOL_KEYOFFING;
 				/*
 				 * Move to the tail of the freelist, unlink it first:
 				 */
@@ -426,6 +433,7 @@ doAudioOps(audioMain *audiomain, float *outbuf, float *startbuf)
 				 */
 				if (voice->baudio->midiflags & BRISTOL_MIDI_DEBUG1)
 					printf("overvoice keydone\n");
+
 				voice->flags |= BRISTOL_KEYOFF;
 				voice = voice->next;
 				continue;
@@ -962,8 +970,8 @@ a440(audioMain *audiomain, float *buf, int count)
 
 	for (obp = 0; obp < count * 2;)
 	{
-		buf[obp++] = sine[(int) index] * 12;
-		buf[obp++] = sine[(int) index] * 12;
+		buf[obp++] = sine[(int) index] * 48;
+		buf[obp++] = sine[(int) index] * 48;
 		if ((index += step) > DCO_WAVE_SZE)
 			index -= DCO_WAVE_SZE;
 	}
