@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -701,6 +701,8 @@ initBristolAudio(audioMain *audiomain, Baudio *baudio)
 {
 	int i, j;
 	bristolOP *op;
+	char name[256];
+	float tmap[128];
 
 	if (baudio == NULL)
 		return;
@@ -878,9 +880,22 @@ initBristolAudio(audioMain *audiomain, Baudio *baudio)
 		}
 	}
 
-	bristolMidiValueMappingTable(baudio->valuemap, baudio->midimap,
+	/*
+	 * We give a NULL name in the engine: these are all GUI mappings but the
+	 * call will init the tables to defaults (linear).
+	 */
+	bristolMidiValueMappingTable(baudio->valuemap, baudio->midimap, "null");
+//		bristolAlgos[(audiomain->midiflags & BRISTOL_PARAMMASK)].name);
+	/* We should do a getmap for the modwheel */
+	sprintf(name, "%s.mcm",
 		bristolAlgos[(audiomain->midiflags & BRISTOL_PARAMMASK)].name);
 
+	if (bristolGetMap(name, "modWheel", tmap, 128, 0) > 0)
+	{
+		for (i = 0; i < 128; i++)
+			baudio->valuemap[1][i] = (int) tmap[i];
+	}
+
 	baudio->midi = bristolMidiController;
 	baudio->mixflags &= ~BRISTOL_HOLDDOWN;
 	baudio->lowkey = 0;
