@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -704,10 +704,6 @@ main(int argc, char **argv)
 	if (audiomain.debuglevel)
 		printf("Got midi thread OK status\n");
 
-#ifdef BRISTOL_SEMAPHORE
-	int exitdecr = 1000000;
-#endif
-
 	while (!exitReq)
 	{
 		/*
