@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -557,7 +557,7 @@ bristolVoice *voice, register float *startbuf)
 		if (mods->flags & ARP_OSC1_LFO)
 		{
 			value = 0.005;
-			mult = 0.40;
+			mult = 0.10;
 		}
 
 		if (abufs.inputs[voice->index][ARP_I_VCO1_KBD] == 
@@ -641,7 +641,7 @@ bristolVoice *voice, register float *startbuf)
 		if (mods->flags & ARP_OSC2_LFO)
 		{
 			value = 0.005;
-			mult = 0.40;
+			mult = 0.10;
 		}
 
 		if (abufs.inputs[voice->index][ARP_I_VCO2_KBD] == 
@@ -725,7 +725,7 @@ bristolVoice *voice, register float *startbuf)
 		if (mods->flags & ARP_OSC3_LFO)
 		{
 			value = 0.005;
-			mult = 0.40;
+			mult = 0.10;
 		}
 
 		if (abufs.inputs[voice->index][ARP_I_VCO3_KBD] == 
@@ -1007,10 +1007,10 @@ bristolVoice *voice, register float *startbuf)
 		 * next available pan buffer as we only use one anyway.
 		 */
 		bufmerge(
-			abufs.inputs[voice->index][ARP_I_LEFT_IN], 1.0,
+			abufs.inputs[voice->index][ARP_I_LEFT_IN], 128.0,
 			abufs.outputs[1][ARP_O_PAN], 1.0, sc);
 		bufmerge(
-			abufs.inputs[voice->index][ARP_I_RIGHT_IN], 1.0,
+			abufs.inputs[voice->index][ARP_I_RIGHT_IN], 128.0,
 			abufs.outputs[2][ARP_O_PAN], 1.0, sc);
 		bufmerge(
 			abufs.inputs[voice->index][ARP_I_PAN], 1.0,
@@ -1099,9 +1099,9 @@ bristolVoice *voice, register float *startbuf)
 		baudio->mixlocals = (float *) voice->locals[voice->index];
 	*/
 
-/* Drive data through the effects: */
 	/*
-	 * We only have the reverb, mono in, stereo out.
+	 * Drive data through the effects:
+	 * We have the reverb, mono in, stereo out. Chorus is later.
 	 */
 	audiomain->palette[(*baudio->sound[13]).index]->specs->io[0].buf =
 		baudio->leftbuf;
@@ -1128,17 +1128,17 @@ bristolVoice *voice, register float *startbuf)
 	 */
 	bufmerge(abufs.outputs[0][ARP_O_PAN],
 		(1.0 - ((arp2600mods *) baudio->mixlocals)->pan) *
-			((arp2600mods *) baudio->mixlocals)->volume,
+			((arp2600mods *) baudio->mixlocals)->volume * 2,
 		baudio->leftbuf,
 		(1.0 - ((arp2600mods *) baudio->mixlocals)->pan) *
-			((arp2600mods *) baudio->mixlocals)->volume,
+			((arp2600mods *) baudio->mixlocals)->volume * 2,
 		audiomain->samplecount);
 	bufmerge(abufs.outputs[0][ARP_O_PAN],
 		((arp2600mods *) baudio->mixlocals)->pan *
-			((arp2600mods *) baudio->mixlocals)->volume,
+			((arp2600mods *) baudio->mixlocals)->volume * 2,
 		baudio->rightbuf,
 		((arp2600mods *) baudio->mixlocals)->pan *
-			((arp2600mods *) baudio->mixlocals)->volume,
+			((arp2600mods *) baudio->mixlocals)->volume * 2,
 		audiomain->samplecount);
 	bufmerge(abufs.outputs[1][ARP_O_PAN], 1.0,
 		baudio->leftbuf, 1.0,
