@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -468,7 +468,7 @@ bristolVoice *voice, register float *startbuf)
 		voice->locals[voice->index][2]);
 /* Osc DONE */
 
-	bufmerge(BME700LOCAL->noisebuf, mixscaler - BME700LOCAL->oscmix,
+	bufmerge(BME700LOCAL->noisebuf, (mixscaler - BME700LOCAL->oscmix) * 32,
 		BME700LOCAL->oscbuf, BME700LOCAL->oscmix, audiomain->samplecount);
 
 /* resonator */
