@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -33,6 +33,7 @@
  * will be an output buffer written to the next operator.
  */
 
+#include <math.h>
 #include "bristol.h"
 #include "envelope.h"
 
@@ -94,6 +95,9 @@ static int param(bristolOP *operator, bristolOPParams *param,
 	unsigned char index, float value)
 {
 	int offset = (value * (CONTROLLER_RANGE - 1)) + 1;
+	bristolENV *specs;
+
+	specs = (bristolENV *) operator->specs;
 
 	if (offset >= CONTROLLER_RANGE)
 		offset = C_RANGE_MIN_1;
@@ -110,14 +114,20 @@ static int param(bristolOP *operator, bristolOPParams *param,
 			 * Attack. This gets tweaked into a rate at operational time as
 			 * per decay and release.
 			 */
-			param->param[index].float_val = value * value;
+			param->param[index].float_val = value;
 			break;
 		case 1:
 		case 3:
 			/*
-			 * Decay and Release.
-			 */
+			 * Decay and Release. We have our overall duration, we need to find
+			 * out how many multiplies we need to decay the target number of
+			 * levels over duration*samplerate samples.
 			param->param[index].float_val = 1/gainTable[offset].rate;
+			 */
+			param->param[index].float_val =
+				1/powf(13.0, 1.0/
+					(2.0 + value*value*value
+						* specs->samplerate*specs->duration));
 			break;
 		case 2:
 			offset = value * (CONTROLLER_RANGE - 1);
@@ -164,6 +174,16 @@ static int param(bristolOP *operator, bristolOPParams *param,
 			 */
 			param->param[index].float_val = value;
 			break;
+		case 9:
+			if ((specs->duration = offset - 1) < 1)
+				specs->duration = 10;
+			break;
+		case 10:
+			if (value != 0)
+				specs->flags |= LINEAR_ATTACK;
+			else
+				specs->flags &= ~LINEAR_ATTACK;
+			break;
 	}
 #ifdef BRISTOL_DBG
 	printf("a %f, d %f, s %0.10f r %f g %f %i, %i\n",
@@ -175,8 +195,7 @@ static int param(bristolOP *operator, bristolOPParams *param,
 	return(0);
 }
 
-#define MM_PARAM voice->velocity * param->param[8].float_val
-/*#define MM_PARAM voice->baudio->contcontroller[1] */
+#define MM_PARAM (1.0f - voice->velocity * param->param[8].float_val) * (1.0f - voice->velocity * param->param[8].float_val)
 /*
  * Pure output signal generator - will drive amps, filters, DCOs, whatever.
  */
@@ -194,24 +213,46 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 	printf("envelope(%x, %x, %x, %x)\n", operator, voice, param, local);
 #endif
 
-	if (param->param[8].float_val == 0.0)
-		attack = gainTable[
-			(int) (C_RANGE_MIN_1 * param->param[0].float_val)].rate;
-	else {
-		/*
-		 * Memory Moog uses this for keyboard controlled envelope. The 'strict'
-		 * specification was that parameters changed by note, not velocity,
-		 * but that was largely because it did not have velocity.
-		 *
-		 * This is going to follow the crumar model - velocity affects only the
-		 * Attack, not decay, sustain or release.
-		 * 
-		 * For the Bit-1/99 this will be changed such that it is not simply
-		 * on or off but scalable.
-		 */
-		attack = param->param[0].float_val * (1.0 - MM_PARAM);
-		attack = gainTable[(int) (attack * C_RANGE_MIN_1 + 1)].rate;
+	if (specs->flags & LINEAR_ATTACK)
+	{
+		if (param->param[8].float_val == 0.0)
+			//attack = gainTable[
+			//	(int) (C_RANGE_MIN_1 * param->param[0].float_val)].rate;
+			attack = 12.0 / (8.0 + specs->duration * specs->samplerate
+				* param->param[0].float_val
+				* param->param[0].float_val
+				* param->param[0].float_val);
+		else
+			/*
+			 * Memory Moog uses this for keyboard controlled envelope. The
+			 * 'strict' specification was that parameters changed by note, not
+			 * velocity, but that was largely because it did not have velocity.
+			 *
+			 * This is going to follow the crumar model - velocity affects only
+			 * the Attack, not decay, sustain or release.
+			 * 
+			 * For the Bit-1/99 this will be changed such that it is not simply
+			 * on or off but scalable.
+			attack = param->param[0].float_val * MM_PARAM;
+			attack = gainTable[(int) (attack * C_RANGE_MIN_1 + 1)].rate;
+			 */
+			attack = 12.0 / (8.0 + specs->duration * specs->samplerate
+				* param->param[0].float_val * MM_PARAM);
+	} else {
+		if (param->param[8].float_val == 0.0)
+			attack = gainTable[
+				(int) (C_RANGE_MIN_1 * param->param[0].float_val
+					* param->param[0].float_val)].rate;
+		else {
+			if ((attack = param->param[0].float_val
+				* param->param[0].float_val * MM_PARAM) > 1.0)
+				attack = 1.0;
+			else if (attack < 0)
+				attack = 0;
+			attack = gainTable[(int) (attack * C_RANGE_MIN_1)].rate;
+		}
 	}
+
 	decay = param->param[1].float_val;
 	sustain = 1.0 + param->param[2].float_val;
 	release = param->param[3].float_val;
@@ -269,9 +310,9 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 
 	if (voice->flags & BRISTOL_KEYON)
 	{
-		/* There is some issue here */
+		/* There is some issue here - do we want to rezero? */
 		if ((voice->baudio->voicecount > 1) || (param->param[6].float_val != 0)
-			|| (cgain == 0.0f))
+			|| (cgain < 1.0f))
 			cgain = 1.0;
 
 		local->cstate = STATE_ATTACK;
@@ -319,7 +360,9 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 		/* Rezero gain */
 		if (param->param[6].float_val != 0)
 			cgain = 1.0;
+
 		local->cstate = STATE_ATTACK;
+		voice->flags &= ~(BRISTOL_KEYDONE|BRISTOL_KEYOFFING);
 	}
 
 	/*
@@ -363,46 +406,74 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 /* printf("keydone: %i, %i, %x\n", voice->key.key, local->cstate, voice); */
 					cgain = 1.0;
 					local->cstate = STATE_DONE;
-					voice->flags |= BRISTOL_KEYDONE;
+					if (~voice->flags & (BRISTOL_KEYON|BRISTOL_KEYREON))
+						voice->flags |= BRISTOL_KEYDONE;
 					voice->flags &= ~BRISTOL_KEYOFFING;
 				} else
-					voice->flags &= ~BRISTOL_KEYDONE;
+					voice->flags &= ~(BRISTOL_KEYDONE);
 				break;
 			case STATE_START:
 			case STATE_ATTACK:
 				/*
 				 * Just triggered - start attack ramp from current gain level.
 				 */
-				while (cgain < BRISTOL_VPO)
-				{
-					if (count-- > 0)
-						*ob++ = ((cgain *= attack) - 1) * egain;
-					else
-						break;
-					if (rampup >= 0) {
-						if (rampup)
-						{
-							if ((egain+=0.01) > gain)
+				if (specs->flags & LINEAR_ATTACK) {
+					while (cgain < BRISTOL_VPO)
+					{
+						if (count-- > 0)
+							*ob++ = ((cgain += attack) - 1) * egain;
+						else
+							break;
+						if (rampup >= 0) {
+							if (rampup)
 							{
-								egain = gain;
-								rampup = -1;
+								if ((egain+=0.01) > gain)
+								{
+									egain = gain;
+									rampup = -1;
+								}
+							} else {
+								if ((egain-=0.01) < gain)
+								{
+									egain = gain;
+									rampup = -1;
+								}
 							}
-						} else {
-							if ((egain-=0.01) < gain)
+						}
+					}
+				} else {
+					while (cgain < BRISTOL_VPO)
+					{
+						if (count-- > 0)
+							*ob++ = ((cgain *= attack) - 1) * egain;
+						else
+							break;
+						if (rampup >= 0) {
+							if (rampup)
 							{
-								egain = gain;
-								rampup = -1;
+								if ((egain+=0.01) > gain)
+								{
+									egain = gain;
+									rampup = -1;
+								}
+							} else {
+								if ((egain-=0.01) < gain)
+								{
+									egain = gain;
+									rampup = -1;
+								}
 							}
 						}
 					}
 				}
+
 				if (cgain >= BRISTOL_VPO) local->cstate = STATE_DECAY;
 				/*
 				 * This just ensures that it is the last envelope in any given
 				 * bristolSound chain that decides when to stop the audio
 				 * stream.
 				 */
-				voice->flags &= ~BRISTOL_KEYDONE;
+				voice->flags &= ~(BRISTOL_KEYDONE);
 				break;
 			case STATE_DECAY:
 				/*
@@ -433,7 +504,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 				}
 				if (cgain <= sustain)
 					local->cstate = STATE_SUSTAIN;
-				voice->flags &= ~BRISTOL_KEYDONE;
+				voice->flags &= ~(BRISTOL_KEYDONE);
 				break;
 			case STATE_SUSTAIN:
 				/*
@@ -460,7 +531,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 						}
 					}
 				}
-				voice->flags &= ~BRISTOL_KEYDONE;
+				voice->flags &= ~(BRISTOL_KEYDONE);
 				break;
 			default:
 				/*
@@ -471,7 +542,9 @@ printf("unknown state: %i, %i\n", voice->key.key, local->cstate);
 				local->cstate = STATE_DONE;
 				while (count-- > 0)
 					*ob++ = 0.0;
-				voice->flags |= BRISTOL_KEYDONE;
+				if (~voice->flags & (BRISTOL_KEYON|BRISTOL_KEYREON))
+					voice->flags |= (BRISTOL_KEYDONE);
+				
 				break;
 		}
 	}
@@ -508,6 +581,11 @@ envinit(bristolOP **operator, int index, int samplerate, int samplecount)
 	(*operator)->param = param;
 
 	specs = (bristolENV *) bristolmalloc0(sizeof(bristolENV));
+	specs->samplerate = samplerate;
+	specs->duration = BRISTOL_RAMP_RATE;
+#ifdef BRISTOL_LIN_ATTACK
+	specs->flags |= LINEAR_ATTACK;
+#endif
 	(*operator)->specs = (bristolOPSpec *) specs;
 	(*operator)->size = sizeof(bristolENV);
 
