@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -29,8 +29,13 @@
 #define STATE_DECAY 12
 #define STATE_SUSTAIN 13
 
+#define LINEAR_ATTACK 1
+
 typedef struct BristolENV {
 	bristolOPSpec spec;
+	float samplerate;
+	float duration;
+	unsigned int flags;
 } bristolENV;
 
 typedef struct BristolENVlocal {
