@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -75,7 +75,9 @@ static float srate;
  * cfreq = cFreq * srate / 1024
  */
 //#define getcoff(c, k) ((c*c)*(1.0f-k)*20000 + k*4*(voice->cFreq * srate / 1024)) / srate
-#define getcoff(c, k) ((c*c)*(1.0f-k)*20000.0f + voice->cFreq*k*srate*0.00390625f) / srate
+//#define getcoff(c, k) ((c*c)*(1.0f-k)*20000.0f + voice->cfreq*k*srate*0.00390625f) / srate
+//#define getcoff(c, k) ((c*c)*(1.0f-k)*20000.0f + k*4*voice->cfreq) / srate
+#define getcoff(c, k) ((c*c)*(1.0f-k)*20000.0f + k*k*20000.0) / srate
 
 //#define TANHF(x) btanhf(voice->baudio->mixflags, x)
 #define TANHF(x) btanhf(mode, x)
@@ -84,9 +86,11 @@ static float srate;
 #define TANHFEED(x) btanhfeed(mode, x)
 //#define TANHFEED tanhf
 //#define TANHFEED(v) (v + 1e-10f) / sqrtf(1 + v * v)
+#define _f_lim (20000.0 / srate)
 
 #define V2 40000.0
 #define OV2 0.000025
+#define F_RESAMPLE 88000
 
 static float
 btanhf(int mode, float v)
@@ -546,7 +550,131 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 		 * Put back the local variables
 		 */
 		local->Bout = Bout;
-	} else if (param->param[4].int_val == 4) {
+	} else if ((param->param[4].int_val == 4) && (srate >= F_RESAMPLE)) {
+		/*
+		 * This is an implementation of Antti Huovilainen's non-linear Moog
+		 * emulation, tweaked just slightly to align with the sometimes rather
+		 * dubious bristol signal levels (its not that they are bad, just that
+		 * historically they have used 1.0 per octave rather than +/-1.0).
+		 *
+		 * This one does not oversample - if the cutoff does not approach
+		 * nyquist then we do not have to correct for its inaccuracies.
+		 */
+		float az1 = local->az1;
+		float az2 = local->az2;
+		float az3 = local->az3;
+		float az4 = local->az4;
+		float az5 = local->az5;
+		float ay1 = local->ay1;
+		float ay2 = local->ay2;
+		float ay3 = local->ay3;
+		float ay4 = local->ay4;
+		float amf = local->amf;
+
+//		float ov2 = 0.000025;// twice the 'thermal voltage of a transistor'
+
+		float kfc;
+		float kfcr;
+		float kacr;
+		float k2vg;
+		float coff;
+
+		float resonance = param->param[1].float_val;
+		Mod = param->param[2].float_val * param->param[2].float_val * 0.002;
+
+		/*
+		 * Cutoff is a power curve for better control at lower frequencies and
+		 * the key is used for tracking purposes, it should be possible to make
+		 * it reasonably linear at somewhere under unity
+		 *
+		 * Cutoff goes from 0 to 1.0 = nyquist. Key tracking should be reviewed,
+		 * the value should be quite linear for the filter, 0..Nyquist
+		 *
+		 * If we take midi key 0 = 8Hz and 127 = 12658.22 Hz then using
+		 * our samplerate we should be able to fix some tuning:
+		 *
+		 * (voice->key.key * 12650.22 + 8) / (127 * srate)
+		 *
+		 * We want to position param[3] such that it tunes at 0.5 and can then
+		 * be notched in the GUI, hence srate/4 rathern than /2.
+		 *
+		 * Needed some changes to key tracking, it was a bit bipolar.
+		coff = (param->param[0].float_val * param->param[0].float_val)
+			* (1.0f - param->param[3].float_val)
+			* 20000 / srate
+				+
+			param->param[3].float_val * 4 * voice->cfreq / srate;
+		 */
+		coff = getcoff(param->param[0].float_val, param->param[3].float_val);
+
+		 /*
+		if (param->param[3].int_val == 0)
+			coff = param->param[0].float_val * param->param[0].float_val
+				* 20000 / srate;
+		else // Fraction of the current frequency by tracking by cutoff.
+			coff = param->param[3].float_val * param->param[0].float_val * 4 *
+				voice->cfreq / srate;
+		 */
+
+//			* voice->cfreq * param->param[3].float_val / srate;
+//printf("%f %f %f\n", voice->cfreq, coff, param->param[3].float_val);
+
+		float lim = _f_lim;
+
+		for (; count > 0; count--)
+		{
+			/*
+			 * The 0.83 is arbitrary, it is used to prevent the filter being
+			 * overdriven however I think 0.5 would be a better value seeing
+			 * as the filter is 2x oversampling?
+			 *
+			 * We should really interpret coff (the configured frequency) as
+			 * a function up to about 20KHz whatever the resampling rate.
+			 */
+			if ((kfc = coff + *mb++ * Mod) > lim)
+				kfc = lim;
+			else if (kfc < 1e-10f)
+				kfc = 1e-10f;
+
+			// frequency & amplitude correction
+			kfcr = kfc * ( kfc * (1.8730 * kfc + 0.4955) - 0.6490) + 0.9988;
+			kacr = kfc * (-3.9364 * kfc + 1.8409) + 0.9968;
+
+			k2vg = (1 - expf(-2.0 * M_PI * kfcr * kfc));
+
+			// cascade of 4 1st order sections
+			dngx1 ^= dngx2;
+			ay1  = az1 + k2vg * (TANHFEED((*ib + dngx2 * scale) * OV2
+				- 4*resonance*amf*kacr) - TANHF(az1));
+			dngx2 += dngx1;
+			az1  = ay1;
+
+			ay2  = az2 + k2vg *(TANHF(ay1) -TANHF(az2));
+			az2  = ay2;
+
+			ay3  = az3 + k2vg * (TANHF(ay2) - TANHF(az3));
+			az3  = ay3;
+
+			ay4  = az4 + k2vg * (TANHF(ay3) - TANHF(az4));
+			az4  = ay4;
+
+			amf  = ay4;
+
+			*ob++ += amf * V2;
+			ib++;
+		}
+
+		local->az1 = az1;
+		local->az2 = az2;
+		local->az3 = az3;
+		local->az4 = az4;
+		local->az5 = az5;
+		local->ay1 = ay1;
+		local->ay2 = ay2;
+		local->ay3 = ay3;
+		local->ay4 = ay4;
+		local->amf = amf;
+	} else if ((param->param[4].int_val == 4) && (srate < F_RESAMPLE)) {
 		/*
 		 * This is an implementation of Antti Huovilainen's non-linear Moog
 		 * emulation, tweaked just slightly to align with the sometimes rather
@@ -573,7 +701,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 		float coff;
 
 		float resonance = param->param[1].float_val;
-		Mod = param->param[2].float_val * param->param[2].float_val * 0.02;
+		Mod = param->param[2].float_val * param->param[2].float_val * 0.002;
 
 		/*
 		 * Cutoff is a power curve for better control at lower frequencies and
@@ -672,7 +800,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 			amf  = (ay4+az5)*0.5;
 			az5  = ay4;
 
-			*ob++ += amf * V2;
+			*ob++ += amf * V2 * 0.5;
 			ib++;
 		}
 
@@ -686,6 +814,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 		local->ay3 = ay3;
 		local->ay4 = ay4;
 		local->amf = amf;
+
 	} else {
 		/* The chamberlain */
 		register float freqcut, highpass, qres,
@@ -707,7 +836,7 @@ static int operate(register bristolOP *operator, bristolVoice *voice,
 
 		qres = 2.0f - param->param[1].float_val * 1.97f;
 		cutoff = param->param[0].float_val;
-		gain = param->param[5].float_val * 0.01565;
+		gain = param->param[5].float_val * 0.02513;
 
 		/*
 		 * The following was for on/off keytracking, needs to be continuous.
