@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -190,15 +190,13 @@ midiControl(audioMain *audiomain, bristolMidiMsg *msg)
 		 * the emulation if it want copies of the controllers. We still put the
 		 * values into our contcontroller structure even if we have a callback.
 		 */
-
-		if ((c_id >= 0) && (c_id < MIDI_CONTROLLER_COUNT))
-			baudio->contcontroller[c_id] = ((float) c_val) / 127;
-		else if (baudio->midiflags & BRISTOL_MIDI_DEBUG1)
-			printf("requested to fill cc %i\n", c_id);
-
 		bristolMidiToGM2(baudio->GM2values, baudio->midimap, baudio->valuemap,
 			msg);
 
+		if ((c_id >= 0) && (c_id < MIDI_CONTROLLER_COUNT))
+//			baudio->contcontroller[c_id] = ((float) c_val) / 127;
+			baudio->contcontroller[c_id] = ((float) msg->params.controller.c_val) / 127.0;
+
 		if (baudio->midiflags & BRISTOL_MIDI_DEBUG1)
 			bristolMidiPrintGM2(msg);
 
@@ -394,10 +392,11 @@ midiPitchWheel(audioMain *audiomain, bristolMidiMsg *msg)
 
  	baudio->pitchwheel = (pitch + 8192) / CONTROLLER_RANGE;
 */
-	baudio->pitchwheel = (pitch / C_RANGE_MIN_1 - 0.5) * 2;
 
 	while ((baudio = findBristolAudioByChan(baudio, msg->channel)) != NULL)
 	{
+		baudio->pitchwheel = (pitch / C_RANGE_MIN_1 - 0.5) * 2;
+
 		if (baudio->midiflags & BRISTOL_MIDI_DEBUG2)
 			bristolMidiPrint(msg);
 
@@ -410,7 +409,7 @@ midiPitchWheel(audioMain *audiomain, bristolMidiMsg *msg)
 		 */
 
 #ifdef DEBUG
-		printf("midiPitchWheel(%f)\n", pitch);
+		printf("midiPitchWheel(%f) %p %i\n", pitch, baudio, msg->channel);
 #endif
 
 		doPitchWheel(baudio);
@@ -643,6 +642,7 @@ initFrequencyTable(float rate)
 	float gain_diff, accum = 1.0;
 
 	samplerate = rate;
+
 	/*
 	 * For any given frequency, we need given number of cycles per second, and
 	 * this is equal to rate/f samples per cycle. We have 1024 samples, so we
@@ -717,9 +717,10 @@ initFrequencyTable(float rate)
 	gain_diff = pow((double) 2, ((double) 1)/CONTROLLER_RANGE);
 	accum = 1.0;
 	for (i = 1; i < CONTROLLER_RANGE; i++)
-		gainTable[i].rate = pow((double) 13, ((double) 1)/(i * 20));
+		gainTable[i].rate = powf(12.0,
+			1.0/(((float) i) * BRISTOL_RAMP_RATE * rate / ((float) C_RANGE_MIN_1)));
 	/* power does not work well for certain values..... */
-	gainTable[0] = gainTable[1];
+	gainTable[0].rate = gainTable[1].rate;
 
 	return(0);
 }
@@ -752,7 +753,7 @@ initMicrotonalTable(float table[])
 void
 initMidiControllerMap(int index, int *map, float *floatmap)
 {
-	int i;
+	int i, c, f;
 	char param[64];
 	float tmpmap[128];
 
@@ -760,16 +761,21 @@ initMidiControllerMap(int index, int *map, float *floatmap)
 	tmpmap[127] = 127.0;
 
 	/* Build a keymap, watch out for failed mappings */
-	if (bristolGetMap("midicontrollermap",
-		&eventNames[index][0], tmpmap, 128, 0) <= 0)
+	if ((c = bristolGetMap("midicontrollermap",
+		&eventNames[index][0], tmpmap, 128, 0)) <= 0)
 		for (i = 0; i < 128; i++)
 			tmpmap[i] = i;
 
-	sprintf(param, "%sFloat", &eventNames[index][0]);
-	bristolGetMap("midicontrollermap", param, floatmap, 128, 0);
-
 	for (i = 0; i < 128; i++)
 		map[i] = (int) tmpmap[i];
+
+	sprintf(param, "%sFloat", &eventNames[index][0]);
+	if (((f = bristolGetMap("midicontrollermap", param, floatmap, 128, 0)) <= 0)
+		&& (c > 0))
+	{
+		for (i = 0; i < 128; i++)
+			floatmap[i] = (float) map[i];
+	}
 }
 
 void
@@ -807,11 +813,13 @@ initMidiRoutines(audioMain *audiomain, midiHandler midiRoutines[])
 	 * affect all emulations.
 	 *
 	 * Continuous controllers are managed separately in the GUI, they are not
-	 * an engine feature.
+	 * an engine feature. We might want to make an exception for ModWheel CC_1?
 	 */
 	for (i = 0; i < 8; i++)
+	{
 		initMidiControllerMap(i, &midiRoutines[i].map[0],
 			&midiRoutines[i].floatmap[0]);
+	}
 
 	if (bristolGetMap("midicontrollermap", "midiNoteMap",
 		&midiRoutines[7].floatmap[0], 128, 0) <= 0)
