@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -155,10 +155,12 @@ doMNL(bristolVoice *voice, int key, int velocity, int action)
 		voice->dfreq = baudio->microtonalmap[mn].freq;
 	} else if (bristolMidiRoutines.bmr[7].floatmap[mn] > 0.0) {
 		/*
-		 * This is the synth global microtonal map
-		 */
+		 * This is the synth private microtonal map (eg, scala file)
 		voice->dFreq = bristolMidiRoutines.freq[mn].step;
 		voice->dfreq = bristolMidiRoutines.freq[mn].freq;
+		 */
+		voice->dFreq = bristolMidiRoutines.bmr[7].floatmap[mn];
+		voice->dfreq = baudio->samplerate * voice->dFreq / ((float) 1024.0);
 	} else {
 		voice->dFreq = baudio->ctab[mn].step;
 		voice->dfreq = baudio->ctab[mn].freq;
@@ -191,7 +193,7 @@ doMNL(bristolVoice *voice, int key, int velocity, int action)
 				voice->velocity = baudio->velocitymap[velocity];
 			}
 
-			voice->flags |= BRISTOL_KEYON;
+			voice->flags |= BRISTOL_KEYON|BRISTOL_KEYREON;
 
 			/* Pro-1 does not glide on first note, if selected */
 			if (baudio->mixflags & BRISTOL_GLIDE_AUTO)
@@ -457,6 +459,10 @@ rbMidiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 	return(0);
 }
 
+static int arbitrary = 0x12365302;
+static int nx1 = 0x67452301;
+static int nx2 = 0xefcdab89;
+
 /*
  * Find a free voice, or take one off the current playlist.
  *
@@ -467,7 +473,7 @@ rbMidiNoteOff(audioMain *audiomain, bristolMidiMsg *msg)
 int
 doMidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 {
-	bristolVoice *voice, *last = NULL;
+	bristolVoice *voice, *last = NULL, *donevoice = NULL;
 	int lastkey, transposedkey, velocity, offset;
 	float cFreq, dFreq, dTune = 0.0, cFreqmult = 0, mappedvelocity;
 	float cfreq, dfreq, cfreqmult;
@@ -529,11 +535,9 @@ doMidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 	 * calling routine, not here.
 	 */
 	if ((baudio->voicecount == 1) && (baudio->lvoices != 0)
+		&& ((voice = audiomain->playlist) != NULL)
 		&& (baudio->notemap.flags & (BRISTOL_MNL_LNP|BRISTOL_MNL_HNP)))
 	{
-		if ((voice = audiomain->playlist) == NULL)
-			return(0);
-
 		while (voice->baudio != baudio)
 			if ((voice = voice->next) == NULL)
 				return(0);
@@ -567,9 +571,11 @@ doMidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 	} else if (bristolMidiRoutines.bmr[7].floatmap[transposedkey] > 0.0) {
 		/*
 		 * This is the synth global microtonal map
-		 */
 		dFreq = bristolMidiRoutines.freq[transposedkey].step;
 		dfreq = bristolMidiRoutines.freq[transposedkey].freq;
+		 */
+		dfreq = bristolMidiRoutines.bmr[7].floatmap[transposedkey];
+		dFreq = ((float) 1024.0) * dfreq / baudio->samplerate;
 	} else {
 		/*
 		 * Otherwise this is the default ET internal map 'currentTable'.
@@ -580,13 +586,17 @@ doMidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 
 	if (baudio->detune != 0)
 	{
-		int arbitrary;
 		/*
 		 * We take a random number. If it is < RAND_MAX/2 then it will be used
 		 * to reduce the frequency by a fraction of the dTune, otherwise it will
 		 * increase the frequency
-		 */
 		arbitrary = rand();
+		 *
+		 * This is a noise generator and is cheaper than rand?
+		 */
+		nx1 ^= nx2;
+		arbitrary += nx2;
+		nx2 += nx1;
 
 		if (arbitrary > (RAND_MAX>>1))
 		{
@@ -762,8 +772,9 @@ doMidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 			 */
 			if (baudio->voicecount == 1)
 			{
-				voice->flags &= ~(BRISTOL_KEYON|BRISTOL_KEYREON);
-					voice->flags |= BRISTOL_KEYREON;
+				voice->flags &= ~(BRISTOL_KEYON|BRISTOL_KEYDONE|BRISTOL_KEYOFF);
+				//|BRISTOL_KEYOFFING);
+				voice->flags |= BRISTOL_KEYREON;
 
 				voice->keyid = key;
 				voice->offset = offset;
@@ -789,6 +800,10 @@ doMidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 				break;
 		}
 
+
+		if (voice->flags & (BRISTOL_DONE|BRISTOL_KEYOFFING))
+			donevoice = voice;
+
 		voice = voice->next;
 	}
 
@@ -797,9 +812,11 @@ doMidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg, Baudio *baudio, int key)
 	 * take the last playlist entry
 	 */
 	if (voice == NULL) {
-		if (last != NULL)
+		if (donevoice != NULL) // Take a voice that is going off
+			voice = donevoice;
+		else if (last != NULL) // Take the last voice for this synth
 			voice = last;
-		else
+		else // Steal the oldest note
 			voice = audiomain->playlast;
 	}
 
@@ -910,7 +927,9 @@ rbMidiNoteOn(audioMain *audiomain, bristolMidiMsg *msg)
 	{
 		msg->command = MIDI_NOTE_OFF|(msg->command & MIDI_CHAN_MASK);
 		msg->params.key.velocity = 64;
+#ifndef BRISTOL_SEMAPHORE
 		return(rbMidiNoteOff(audiomain, msg));
+#endif
 	}
 
 	/*
@@ -1027,8 +1046,8 @@ allNotesOff(audioMain *audiomain, int channel)
 				&& ((voice->baudio->midichannel == channel)
 					|| (voice->baudio->midichannel == BRISTOL_CHAN_OMNI))))
 		{
-			voice->flags |= BRISTOL_KEYOFF|BRISTOL_KEYDONE;
-			voice->flags &= ~BRISTOL_KEYSUSTAIN;
+			voice->flags = BRISTOL_KEYOFF|BRISTOL_KEYDONE;
+			//voice->flags &= ~(BRISTOL_KEYSUSTAIN;
 		}
 
 		doMNL(voice, 0, 0, BRISTOL_ALL_NOTES_OFF);
