@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,11 @@
  * Reresolved sample delta so that PWM itself is resampled.
  */
 
+/*
+ * The 'param' is not per operator local memory, it is per voice local memory
+ * and we need to review how this is being used.
+ */
+#pragma param structures need to be revisited
 #include <math.h>
 
 #include "bristol.h"
@@ -223,10 +228,11 @@ static int pdcoparam(bristolOP *operator, bristolOPParams *param,
 				param->param[index].int_val = 0;
 			else
 				param->param[index].int_val = 1;
-			buildProphetWave(param->param[0].mem,
-				param->param[8].int_val, specs->wave[7],
-				param->param[4].int_val, specs->wave[3],
-				param->param[5].int_val, specs->wave[4]);
+			specs->flags |= PROPHETDCO_REWAVE;
+//			buildProphetWave(param->param[0].mem,
+//				param->param[8].int_val, specs->wave[7],
+//				param->param[4].int_val, specs->wave[3],
+//				param->param[5].int_val, specs->wave[4]);
 			break;
 		case 6: /* mix PWM square */
 			if (value == 0)
@@ -320,8 +326,7 @@ buildProphetWave(register float *wave,
 {
 	register int i;
 
-/*	printf("buildProphetWave(%i, %i, %i)\n", sine, ramp, tri); */
-/*	printf("buildProphetWave(%x, %x, %x)\n", sinew, rampw, triwave); */
+//printf("buildProphetWave(%p, %p, %p)->%p\n", sinew, rampw, triwave, wave);
 
 	bristolbzero(wave, sizeof(float) * PROPHETDCO_WAVE_SZE);
 
@@ -388,7 +393,7 @@ static int operate(bristolOP *operator,
 	specs = (bristolPROPHETDCO *) operator->specs;
 
 #ifdef BRISTOL_DBG
-	printf("prophetdco(%x, %x, %x)\n", operator, param, local);
+	printf("prophetdco(%p, %p, %p)\n", operator, param, local);
 #endif
 
 	count = specs->spec.io[DCO_OUT_IND].samplecount;
@@ -412,7 +417,15 @@ static int operate(bristolOP *operator,
 		param->param[5].int_val, specs->wave[4]);
 	*/
 
-	wt = (float *) param->param[0].mem;
+	if ((wt = local->wave0) == NULL)
+	{
+		local->wave0 =
+			(float *) bristolmalloc(PROPHETDCO_WAVE_SZE * sizeof(float));
+		local->wave1 =
+			(float *) bristolmalloc(PROPHETDCO_WAVE_SZE * sizeof(float));
+		local->wave2 =
+			(float *) bristolmalloc(PROPHETDCO_WAVE_SZE * sizeof(float));
+	}
 	width = param->param[0].float_val; /* Pulse width */
 	dosquare = param->param[6].int_val;
 /*	wtp = local->wtp; */
@@ -449,25 +462,37 @@ blo.harmonics, voice->key.key * transp < 95?
 			(int) ( blo.harmonics - (voice->key.key - 95) * transp * 3));
 	 */
 
-	if (bristolBLOcheck(voice->dFreq*transp))
+	if ((specs->flags & PROPHETDCO_REWAVE)
+		|| (voice->flags & (BRISTOL_KEYON|BRISTOL_KEYREON)))
 	{
-		memset(param->param[1].mem, 0, PROPHETDCO_WAVE_SZE * sizeof(float));
-		memset(param->param[2].mem, 0, PROPHETDCO_WAVE_SZE * sizeof(float));
-		wt = (float *) param->param[1].mem;
-		ramp = (float *) param->param[2].mem;
-		if (param->param[4].int_val) {
-			/* Put in BLO ramp wave, copy to PWM wave */
-			generateBLOwaveformF(voice->cFreq*transp, wt, BLO_RAMP);
-			bcopy(wt, ramp, PROPHETDCO_WAVE_SZE * sizeof(float));
-		} else {
-			/* Put in ramp wave for PWM */
-			if (param->param[6].int_val)
-				generateBLOwaveformF(voice->cFreq*transp, ramp, BLO_RAMP);
-		}
-		if (param->param[5].int_val)
-			generateBLOwaveformF(voice->cFreq*transp, wt, BLO_TRI);
-		if (param->param[8].int_val)
-			generateBLOwaveformF(voice->cFreq*transp, wt, BLO_SINE);
+//		if (bristolBLOcheck(voice->dFreq*transp))
+//		{
+//			memset(param->param[1].mem, 0, PROPHETDCO_WAVE_SZE * sizeof(float));
+//			memset(param->param[2].mem, 0, PROPHETDCO_WAVE_SZE * sizeof(float));
+			memset(local->wave1, 0, PROPHETDCO_WAVE_SZE * sizeof(float));
+			memset(local->wave2, 0, PROPHETDCO_WAVE_SZE * sizeof(float));
+			wt = (float *) local->wave1;
+			ramp = (float *) local->wave2; //param->param[2].mem;
+			if (param->param[4].int_val) {
+				/* Put in BLO ramp wave, copy to PWM wave */
+				generateBLOwaveformF(voice->dFreq*transp, wt, BLO_RAMP);
+				bcopy(wt, ramp, PROPHETDCO_WAVE_SZE * sizeof(float));
+			} else {
+				/* Put in ramp wave for PWM */
+				if (param->param[6].int_val)
+					generateBLOwaveformF(voice->dFreq*transp, ramp, BLO_RAMP);
+			}
+			if (param->param[5].int_val)
+				generateBLOwaveformF(voice->dFreq*transp, wt, BLO_TRI);
+			if (param->param[8].int_val)
+			generateBLOwaveformF(voice->dFreq*transp, wt, BLO_SINE);
+//		}
+
+		buildProphetWave(local->wave0, //param->param[0].mem,
+			param->param[8].int_val, specs->wave[7],
+			param->param[4].int_val, specs->wave[3],
+			param->param[5].int_val, specs->wave[4]);
+		specs->flags &= ~PROPHETDCO_REWAVE;
 	}
 
 	/*
