@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol audio routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -28,13 +28,15 @@
 #define PROPHETDCO_PW_MIN		50
 #define PROPHETDCO_PW_MAX		975
 
-#define PROPHETDCO_SYNC 0x01
+#define PROPHETDCO_SYNC		0x01
+#define PROPHETDCO_REWAVE	0x02
 
 #define S_DEC 0.999f
 
 typedef struct BristolPROPHETDCO {
 	bristolOPSpec spec;
 	float *wave[8];
+	unsigned int flags;
 } bristolPROPHETDCO;
 
 typedef struct BristolPROPHETDCOlocal {
@@ -45,6 +47,7 @@ typedef struct BristolPROPHETDCOlocal {
 	float lsv;
 	float ssg;
 	int tune_diff;
+	float *wave0, *wave1, *wave2;
 } bristolPROPHETDCOlocal;
 
 #endif /* PROPHETDCO_H */
