@@ -1,7 +1,7 @@
 
 /*
  *  Diverse Bristol midi routines.
- *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2010
+ *  Copyright (c) by Nick Copeland <nickycopeland@hotmail.com> 1996,2011
  *
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -281,6 +281,7 @@ bristolGetMap(char *file, char *match, float *points, int count, int flags)
 				{
 					points[wheel] = value;
 					mapped++;
+					//printf("mapped %i to %0.2f\n", wheel, value);
 				}
 			}
 		}
@@ -328,12 +329,14 @@ bristolGetMap(char *file, char *match, float *points, int count, int flags)
 		points[n] = points[n - 1];
 
 /*
+if (mapped)
+{
 	i = 0;
 	printf("%s\n", match);
 	printf("%1.2f %1.2f %1.2f %1.2f %1.2f %1.2f %1.2f %1.2f\n",
 		points[i], points[i+1], points[i+2], points[i+3], 
 		points[i+4], points[i+5], points[i+6], points[i+7]);
-	if ((i += 8) < count) 
+	if ((i += 8) < 128) 
 		printf("%1.2f %1.2f %1.2f %1.2f %1.2f %1.2f %1.2f %1.2f\n",
 			points[i], points[i+1], points[i+2], points[i+3], 
 			points[i+4], points[i+5], points[i+6], points[i+7]);
@@ -376,6 +379,7 @@ bristolGetMap(char *file, char *match, float *points, int count, int flags)
 	if ((i += 8) < count) 
 		printf("%1.2f %1.2f %1.2f\n",
 			points[i], points[i+1], points[i+2]);
+}
 */
 
 	return(mapped);
